@@ -1,2 +1,2 @@
 from .table_evaluator import TableEvaluator
-from .helpers import *
\ No newline at end of file
+from .utils import load_data
